# Code Mixed Text Toolkit (CMTT)

"""
Code Mixed Text Toolkit (CMTT) is a wrapper library that makes code-mixed text processing more efficient than ever.
"""

from cmtt.data import *
from cmtt.preprocessing import *
from cmtt.LID import *